@@ -2,8 +2,18 @@
 # Environment env is used to pass some useful data
 # (such as last solution and Cholesky decomposition)
 # between different calls of lmSolver.
-lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, iterControl = list(maxiter = 20, tol = 1E-6), trace = TRUE) {
-  if (length(y) > nrow(X)) stop("y is too long in lmSolver...")
+lmSolver <- function(
+  X,
+  y,
+  type = "Matrix",
+  method = "cholesky",
+  env = NULL,
+  iterControl = list(maxiter = 20, tol = 1E-6),
+  trace = TRUE
+) {
+  if (length(y) > nrow(X)) {
+    stop("y is too long in lmSolver...")
+  }
   y <- c(y, rep(0, nrow(X) - length(y)))
   if (type == "Matrix") {
     if (method == "cholesky") {
@@ -19,11 +29,19 @@ lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, ite
   if (type == "iterative") {
     if (method == "cg") {
       f <- function(z) crossprod(X, X %*% z)
-      if (is.null(env) || is.null(env$L) || is.null(env$Lt) || is.null(env$P) || is.null(env$Pt)) {
+      if (
+        is.null(env) ||
+          is.null(env$L) ||
+          is.null(env$Lt) ||
+          is.null(env$P) ||
+          is.null(env$Pt)
+      ) {
         invm <- 1 / colSums(X^2)
         invf <- function(z) invm * z
       } else {
-        invf <- function(z) solve(env$P, solve(env$Lt, solve(env$L, solve(env$Pt, z))))
+        invf <- function(z) {
+          solve(env$P, solve(env$Lt, solve(env$L, solve(env$Pt, z))))
+        }
       }
       Xty <- crossprod(X, y)
       if (is.null(env) || is.null(env$b0)) {
@@ -31,17 +49,26 @@ lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, ite
       } else {
         b0 <- env$b0
       }
-      result <- olscg(FUN = f, y = Xty, b = b0, invFUN = invf, iterControl = iterControl, trace = trace)
+      result <- olscg(
+        FUN = f,
+        y = Xty,
+        b = b0,
+        invFUN = invf,
+        iterControl = iterControl,
+        trace = trace
+      )
       return(result$b)
     }
     if (method == "cg-chol") {
       if (is.null(env)) {
         stop("env variable is NULL.")
       }
-      if (is.null(env$L) ||
-        is.null(env$Lt) ||
-        is.null(env$P) ||
-        is.null(env$Pt)) {
+      if (
+        is.null(env$L) ||
+          is.null(env$Lt) ||
+          is.null(env$P) ||
+          is.null(env$Pt)
+      ) {
         result <- .solve.dgC.chol(t(X), y)
         eL <- Matrix::expand(result$L)
         env$L <- eL$L
@@ -52,14 +79,23 @@ lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, ite
         return(result$coef)
       } else {
         f <- function(z) crossprod(X, X %*% z)
-        invf <- function(z) solve(env$P, solve(env$Lt, solve(env$L, solve(env$Pt, z))))
+        invf <- function(z) {
+          solve(env$P, solve(env$Lt, solve(env$L, solve(env$Pt, z))))
+        }
         Xty <- crossprod(X, y)
         if (is.null(env$b0)) {
           b0 <- rep(0, length(Xty))
         } else {
           b0 <- env$b0
         }
-        result <- olscg(FUN = f, y = Xty, b = b0, invFUN = invf, iterControl = iterControl, trace = trace)
+        result <- olscg(
+          FUN = f,
+          y = Xty,
+          b = b0,
+          invFUN = invf,
+          iterControl = iterControl,
+          trace = trace
+        )
         return(result$b)
       }
     }
@@ -67,10 +103,12 @@ lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, ite
       if (is.null(env)) {
         stop("env variable is NULL.")
       }
-      if (is.null(env$L) ||
-        is.null(env$Lt) ||
-        is.null(env$P) ||
-        is.null(env$Pt)) {
+      if (
+        is.null(env$L) ||
+          is.null(env$Lt) ||
+          is.null(env$P) ||
+          is.null(env$Pt)
+      ) {
         result <- .solve.dgC.chol(t(X), y)
         eL <- Matrix::expand(result$L)
         env$L <- eL$L
@@ -92,7 +130,13 @@ lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, ite
         } else {
           x0 <- 0
         }
-        result <- lsmr(A = A, b = y - X %*% x0, atol = iterControl$tol, btol = iterControl$tol, itnlim = iterControl$maxiter)
+        result <- lsmr(
+          A = A,
+          b = y - X %*% x0,
+          atol = iterControl$tol,
+          btol = iterControl$tol,
+          itnlim = iterControl$maxiter
+        )
         if (trace) {
           cat("\nIterations: ")
           cat(result$itn)
@@ -116,7 +160,13 @@ lmSolver <- function(X, y, type = "Matrix", method = "cholesky", env = NULL, ite
       } else {
         x0 <- rep(0, ncol(X))
       }
-      result <- lsmr(A = A, b = y - X %*% x0, atol = iterControl$tol, btol = iterControl$tol, itnlim = iterControl$maxiter)
+      result <- lsmr(
+        A = A,
+        b = y - X %*% x0,
+        atol = iterControl$tol,
+        btol = iterControl$tol,
+        itnlim = iterControl$maxiter
+      )
       if (trace) {
         cat("\nIterations: ")
         cat(result$itn)
